import sys
from os import*
import os
from os.path import*
from Tkinter import*
import tkMessageBox
from topowidget import*
from threading import*
from thread import*
import imagetopo
import controldata
import helixwindow


class Gui(object): # main application window
	def __init__(self):
		self.mainwindow = Tk()
		if self.mainwindow.winfo_screenheight() > 900:
			self.mainwindow.geometry('800x800')
		else:
			self.mainwindow.geometry('800x680')
		self.mainwindow.resizable(False,False)
		self.mainwindow.protocol("WM_DELETE_WINDOW", self.callback)
		self.mainwindow.title('TopoView')
		
		self.scrollbar1 = Scrollbar(self.mainwindow)
		self.scrollbar2 = Scrollbar(self.mainwindow,orient=HORIZONTAL)
		self.scrollbar1.pack(side=RIGHT,fill=Y)
		self.scrollbar2.pack(side=BOTTOM,fill=X)
		
		self.showroom = Canvas(self.mainwindow,width=800,height=800,bg='white',scrollregion=(0,0,800,800),yscrollcommand=self.scrollbar1.set,xscrollcommand=self.scrollbar2.set)
		self.showroom.pack(fill=BOTH)

		self.scrollbar1.config(command=self.showroom.yview)
		self.scrollbar2.config(command=self.showroom.xview)
		
		self.path = getcwd()
		self.idir = self.path +'/images'
		self.mainpath = abspath('..')
		self.image = 'topoprotein.ppm'
		self.topofile = ''
		
		self.startimage = 'topoimage.ppm'
		self.topo = PhotoImage(file=self.path+'/'+self.startimage)
		self.showroom.create_image(0,0,anchor=NW,image=self.topo)
		
		self.menubar = Menu(self.mainwindow)
		self.filemenu = Menu(self.menubar)
		self.filemenu.add_command(label = 'Create topology',command=self.createtopo)
		self.filemenu.add_command(label = 'Load topology',command=self.load)
		self.filemenu.add_command(label = 'Load TMX prediction',command=self.loadtmx, state=DISABLED)
		self.filemenu.add_command(label = 'Export image (png)',command=self.export, state=DISABLED)
		self.filemenu.add_command(label = 'Export highquality image (png)',command=self.export1, state=DISABLED)
		self.menubar.add_cascade(label='File', menu=self.filemenu)
		
		self.optionmenu = Menu(self.menubar)
		self.optionmenu.add_command(label = 'Zoom in', command=self.zoomin, state=DISABLED)
		self.optionmenu.add_command(label = 'Zoom out', command=self.zoomout, state=DISABLED)
		self.optionmenu.add_command(label = 'Mark non contact', command=self.marknc, state=DISABLED)
		self.optionmenu.add_command(label = 'Unmark non contact', command=self.unmarknc, state=DISABLED)
		self.menubar.add_cascade(label='Options', menu=self.optionmenu)
		
		self.helixmenu = Menu(self.menubar)
		self.helixmenu.add_command(label = 'Poperty coloring', command=self.genhelixwheel_property, state=DISABLED)
		self.helixmenu.add_command(label = 'TMX coloring', command=self.genhelixwheel_tmx, state=DISABLED)
		self.helixmenu.add_command(label = 'Conservation coloring', command=self.genhelixwheel_cons, state=DISABLED)
		self.menubar.add_cascade(label='Helical wheels', menu=self.helixmenu, state=DISABLED)
		
		
		self.helpmenu = Menu(self.menubar)
		self.helpmenu.add_command(label = 'Help', command=self.help)
		self.helpmenu.add_command(label = 'About', command=self.about)
		self.menubar.add_cascade(label='Help', menu=self.helpmenu)
		
		self.mainwindow.config(menu=self.menubar)
		self.mainwindow.mainloop()
		
	
	
	def createtopo (self): # opens the creation window, after topology is calculted buttons were enabled and other images were calculated
		
		self.refreshdata()
		
		p = popen('python startcreate.py', 'r')
		p.close()
		
		self.topofile = self.idir+'/topology.txt'
		
		try:
			t = access(self.idir+'/topoprotein.pov',F_OK)
		except OSError:
			pass
		if t:
			povimage = self.idir+'/topoprotein.pov'
			name = self.idir+'/topoproteinmax'
			w = 1800
			self.tp1 = start_new_thread(imagetopo.exe_povray,(povimage,name,w))
			
			self.image = 'topoprotein.ppm'
			self.topo.config(file=self.idir+'/'+self.image)
			self.showroom.create_image(0,0,anchor=NW,image=self.topo)
			
			file_lst = listdir(self.idir)
			self.hw_lst = []
			
			for f in file_lst:
				if f.startswith('helixwheel') and not f.endswith('_1.pov'):
					self.hw_lst.append(f)
			
			for hw_file in self.hw_lst:
				povimage = self.idir+'/'+hw_file
				name = self.idir+'/'+hw_file[:len(hw_file)-4]
				w = 600
				start_new_thread(imagetopo.exe_povray,(povimage,name,w))
			
			self.optionmenu.entryconfigure(2,state=NORMAL)
			self.menubar.entryconfigure(3,state=NORMAL)
			self.helixmenu.entryconfigure(1,state=NORMAL)
			self.filemenu.entryconfigure(3,state=NORMAL)
			self.filemenu.entryconfigure(4,state=NORMAL)
			self.filemenu.entryconfigure(5,state=NORMAL)
			self.optionmenu.entryconfigure(1,state=NORMAL)
			
		try:
			t = access(self.idir+'/topoproteinnc.pov',F_OK)
		except OSError:
			pass
		if t:	
			povimage1 = self.idir+'/topoproteinnc.pov'
			name1 = self.idir+'/topoproteinnc'
			w1 = 800
			self.tp2 = start_new_thread(imagetopo.exe_povray,(povimage1,name1,w1))
			povimage2 = self.idir+'/topoproteinnc.pov'
			name2 = self.idir+'/topoproteinncmax'
			w2 = 1800
			self.tp3 = start_new_thread(imagetopo.exe_povray,(povimage2,name2,w2))
			self.optionmenu.entryconfigure(3,state=NORMAL)
			self.optionmenu.entryconfigure(4,state=NORMAL)
	
	
	
	def load(self): # loads a topology file
		self.filepath = tkFileDialog.askopenfilename(title='Open topology file')
		if self.filepath == '':
			pass
		else:
			self.topofile = self.filepath
		file_obj = file(self.filepath, 'r')
		lines = file_obj.readlines()
		file_obj.close()
		
		faults = controldata.controltopology(lines) # modul controldata is called
		
		if len(faults) == 0:
			
			self.refreshdata()
			
			t1=Timer(0.2,self.itopo) # starts after 0.2 sec the function topo()
			t1.start()
			
			povimage= self.idir+'/topoprotein.pov'
			args = [povimage]
			t2 = Timer(2,self.ipovi,args) # starts after 2 secs the function povi()
			t2.start()
			
			t3=Timer(10,self.displayimage) # starts after 10 sec the function displayimage()
			t3.start()
			tkMessageBox.showinfo('Please wait...','Computations running')
		else:
			for f in faults:
				if tkMessageBox.showwarning('Load Problems',f) == 'ok':
					continue

	
	def displayimage(self): # if a topology file is uploaded this function enables the display of the topology image
		
		try:
			t = access(self.idir+'/topoprotein.pov',F_OK)
		except OSError:
			pass
		if t:
			povimage = self.idir+'/topoprotein.pov'
			name = self.idir+'/topoproteinmax'
			w = 1800
			self.tp1 = start_new_thread(imagetopo.exe_povray,(povimage,name,w))
			self.image = 'topoprotein.ppm'
			self.topo.config(file=self.idir+'/'+self.image)
			self.showroom.create_image(0,0,anchor=NW,image=self.topo)
			self.showroom.config(scrollregion=(0, 0, 800, 800))
			
			file_lst = listdir(self.idir)
			self.hw_lst = [] # list of helical wheels
			
			for f in file_lst:
				if f.startswith('helixwheel') and not f.endswith('_1.pov'):
					self.hw_lst.append(f)
			
			
			for hw_file in self.hw_lst: # all helical wheels are rendered
				povimage = self.idir+'/'+hw_file
				name = self.idir+'/'+hw_file[:len(hw_file)-4]
				w = 600
				start_new_thread(imagetopo.exe_povray,(povimage,name,w))
			
			
			self.optionmenu.entryconfigure(2,state=NORMAL)
			self.menubar.entryconfigure(3,state=NORMAL)
			self.helixmenu.entryconfigure(1,state=NORMAL)
			self.filemenu.entryconfigure(3,state=NORMAL)
			self.filemenu.entryconfigure(4,state=NORMAL)
			self.filemenu.entryconfigure(5,state=NORMAL)
			self.optionmenu.entryconfigure(1,state=NORMAL)
		try:
			t = access(self.idir+'/topoproteinnc.pov',F_OK)
		except OSError:
			pass
		if t:	
			povimage1 = self.idir+'/topoproteinnc.pov'
			name1 = self.idir+'/topoproteinnc'
			w1 = 800
			self.tp2 = start_new_thread(imagetopo.exe_povray,(povimage1,name1,w1))
			povimage2 = self.idir+'/topoproteinnc.pov'
			name2 = self.idir+'/topoproteinncmax'
			w2 = 1800
			self.tp3 = start_new_thread(imagetopo.exe_povray,(povimage2,name2,w2))
			self.optionmenu.entryconfigure(3,state=NORMAL)
			self.optionmenu.entryconfigure(4,state=NORMAL)


	def itopo(self): # modul imagetopo is called and writes a povray file
		imagetopo.createtopo(self.filepath,self.idir+'/topoprotein',self.idir+'/helixwheel') # modul imagetopo is called and writes a povray file	
	
	
	def ipovi(self,pi): # modul imagetopo is called and the ray tracer povray parses the pov file
		name = self.idir+'/topoprotein'
		w = 800
		imagetopo.exe_povray(pi,name,w)
	
	
	def loadtmx (self): #for uploading a tmx predition file, so that the tmx and conservation coloring is possible
		self.filepath_tmx = tkFileDialog.askopenfilename(title='Open TMX file')
		tmx_file_obj = file(self.filepath_tmx, 'r')
		self.tmx_lines = tmx_file_obj.readlines()
		tmx_file_obj.close()
		
		tmx_faults = controldata.controltmx(self.tmx_lines) # modul controldata is called
		
		if len(tmx_faults) == 0:
			
			tmx_t1=Timer(0.2,self.itopo_tmx) # starts after 0.2 sec the function itopo_tmx()
			tmx_t1.start()
			
			tmx_t2=Timer(2,self.ipovi_tmx) # starts after 2 sec the function ipovi_tmx()
			tmx_t2.start()
			
			self.helixmenu.entryconfigure(2,state=NORMAL)
			self.helixmenu.entryconfigure(3,state=NORMAL)
			
		else:
			for ftmx in tmx_faults:
				if tkMessageBox.showwarning('Load Problems',ftmx) == 'ok':
					continue
	
	
	def itopo_tmx(self): # modul imagetopo is called and writes a povray files for helical wheels
		imagetopo.createtmxdata(self.filepath_tmx,self.topofile, self.idir+'/helixwheeltmx', self.idir+'/helixwheelcons') # modul imagetopo is called and writes a povray file	
	
	
	def ipovi_tmx(self): # modul imagetopo is called and the ray tracer povray parses the pov file
		
		image_lst = listdir(self.idir)
		self.hwtmx_lst = []
		self.hwcons_lst = []
		
		for i in image_lst:
			if i.startswith('helixwheeltmx') and not i.endswith('_1.pov'):
				self.hwtmx_lst.append(i)
			elif i.startswith('helixwheelcons') and not i.endswith('_1.pov'):
				self.hwcons_lst.append(i)
		
		for hwtmx_file in self.hwtmx_lst:
			povimage = self.idir+'/'+hwtmx_file
			name = self.idir+'/'+hwtmx_file[:len(hwtmx_file)-4]
			w = 600
			start_new_thread(imagetopo.exe_povray,(povimage,name,w))
	
		for hwcons_file in self.hwcons_lst:
			povimage = self.idir+'/'+hwcons_file
			name = self.idir+'/'+hwcons_file[:len(hwcons_file)-4]
			w = 600
			start_new_thread(imagetopo.exe_povray,(povimage,name,w))

	
	
	def  export (self): #export image as low resolution png-image
		try:
			t = access(self.idir+'/'+self.image,F_OK)
		except OSError:
			pass
		if t:
			place = tkFileDialog.asksaveasfilename(title='Save file as...')
			if place != ():
				if self.image[len(self.image)-5] == 'x':
					povimage = self.image[:len(self.image)-7] +'1.pov'
				else:
					povimage = self.image[:len(self.image)-4] +'1.pov'
				name = place.split('/')
				n = name[len(name)-1]
				p = '/'.join(name[:len(name)-1])
				if ' ' in p:
					p = p.replace(' ','\ ')
				w = 1000
				cmd = 'povray  '+self.idir+'/'+povimage+' +O'+self.idir+'/'+n+' +W'+str(w)+' +H'+str(w)+' -D +A'
				popen(cmd, 'r')
				cmd1 = 'mv '+self.idir+'/'+n+'.png '+p+'/'
				popen(cmd1, 'r')
		else:
			tkMessageBox.showwarning('Export Problems','You first have to create a topology!')
	
	
	def  export1 (self): #export image as high resolution png-image
		try:
			t = access(self.idir+'/'+self.image,F_OK)
		except OSError:
			pass
		if t :
			place = tkFileDialog.asksaveasfilename(title='Save file as...')
			if place != ():
				if self.image[len(self.image)-5] == 'x':
					povimage = self.image[:len(self.image)-7] +'1.pov'
				else:
					povimage = self.image[:len(self.image)-4] +'1.pov'
				name = place.split('/')
				n = name[len(name)-1]
				p = '/'.join(name[:len(name)-1])
				if ' ' in p:
					p = p.replace(' ','\ ')
				w = 1800
				cmd = 'povray  '+self.idir+'/'+povimage+' +O'+self.idir+'/'+n+' +W'+str(w)+' +H'+str(w)+' -D +A'
				popen(cmd, 'r')
				cmd1 = 'mv '+self.idir+'/'+n+'.png '+p+'/'
				popen(cmd1, 'r')
		else:
			tkMessageBox.showwarning('Export Problems','You first have to create a topology!')


	
	def  zoomin (self): # increases the image about 100%
		s = 0
		try:
			 t = access(self.idir+'/'+self.image[:len(self.image)-4]+'max.ppm',F_OK)
		except OSError:
			pass
		if t :
			try:
				PhotoImage(file=self.idir+'/'+self.image[:len(self.image)-4]+'max.ppm')
				s = 1
			except TclError:
				pass
			if s:
				self.topo.config(file=self.idir+'/'+self.image[:len(self.image)-4]+'max.ppm')
				self.showroom.create_image(0,0,anchor=NW,image=self.topo)
				self.showroom.config(scrollregion=(0, 0, 1800, 1800))
			else:
				tkMessageBox.showwarning('Calculations running...','Please try again in a few seconds!')
		else:
			tkMessageBox.showwarning('Calculations running...','Please try again in a few seconds!')

	
	
	def  zoomout (self): # decreases the image to general view
		try:
			t = access(self.idir+'/'+self.image,F_OK)
		except OSError:
			pass
		if t:
			self.topo.config(file=self.idir+'/'+self.image)
			self.showroom.create_image(0,0,anchor=NW,image=self.topo)
			self.showroom.config(scrollregion=(0, 0, 800, 800))
		else:
			tkMessageBox.showwarning('Calculations running...','Please try again in a few seconds!')
	
	
	def  marknc (self): # if non contacts were specified they get marked
		s = 0
		try:
			t = access(self.idir+'/topoproteinnc.ppm',F_OK)
		except OSError:
			pass
		if t:
			try:
				PhotoImage(file=self.idir+'/topoproteinnc.ppm')
				s = 1
			except TclError:
				pass
			if s:
				self.image = 'topoproteinnc.ppm'
				self.topo.config(file=self.idir+'/'+self.image)
				self.showroom.create_image(0,0,anchor=NW,image=self.topo)
				self.showroom.config(scrollregion=(0, 0, 800, 800))
			else:
				tkMessageBox.showwarning('Calculations running...','Please try again in a few seconds!')
		else:
			tkMessageBox.showwarning('Calculations running...','Please try again in a few seconds!')

	
	
	def  unmarknc (self): # unmark the non contacts if they where maked before
		try:
			t = access(self.idir+'/topoprotein.ppm',F_OK)
		except OSError:
			pass
		if t:
			self.image = 'topoprotein.ppm'
			self.topo.config(file=self.idir+'/'+self.image)
			self.showroom.create_image(0,0,anchor=NW,image=self.topo)
			self.showroom.config(scrollregion=(0, 0, 800, 800))
		else:
			tkMessageBox.showwarning('Calculations running...','Please try again in a few seconds!')
	
	
	def refreshdata(self): # if a new topology is created all the old files were deleted
		
		self.showroom.config(scrollregion=(0, 0, 800, 800))
		self.optionmenu.entryconfigure(2,state=DISABLED)
		self.filemenu.entryconfigure(3,state=DISABLED)
		self.filemenu.entryconfigure(4,state=DISABLED)
		self.optionmenu.entryconfigure(1,state=DISABLED)
		self.optionmenu.entryconfigure(3,state=DISABLED)
		self.optionmenu.entryconfigure(4,state=DISABLED)
		self.helixmenu.entryconfigure(1,state=DISABLED)
		self.helixmenu.entryconfigure(1,state=DISABLED)
		self.helixmenu.entryconfigure(1,state=DISABLED)
		
		self.image = 'load.gif'
		self.topo.config(file=self.path+'/'+self.image)
		self.showroom.create_image(0,0,anchor=NW,image=self.topo)
		self.showroom.config(scrollregion=(0, 0, 800, 800))
		
		allcreatedfiles = listdir(self.idir)
			
		for f in allcreatedfiles:
			if f != 'DejaVuSans.ttf':
				remove(self.idir+'/'+f)

	
	
	def genhelixwheel_property(self):# starts the helixwheel window with property coloring
		self.genhelixwheel('helixwheel1.ppm')
	
	def genhelixwheel_tmx(self):# starts the helixwheel window with tmx coloring
		self.genhelixwheel('helixwheeltmx1.ppm')
		
	def genhelixwheel_cons(self):# starts the helixwheel window with conservation coloring
		self.genhelixwheel('helixwheelcons1.ppm')
	
	
	def genhelixwheel(self,hwname):# calls the helixwheel window
		helixwindow.start(hwname)
	
	
	def  help (self): # open the manual pdf
		setup = []
		try:
			t = access(self.mainpath+'/SETUP',F_OK)
			if t:
				setupfile = file(self.mainpath+'/SETUP','r')
				setup = setupfile.readlines()
		except OSError:
			pass

		if setup != []:
			pdfpath = ''
			pdfname = ''
			for lines in setup:
				if lines.startswith('path ='):
					l = lines.split('=')
					pdfpath = lstrip(rstrip(l[1]))
					pd = pdfpath.split('/')
					pdfname = pd[len(pd)-1]
			if pdfpath != '' and pdfname != '':
				os.spawnl(os.P_NOWAIT, pdfpath, pdfname, self.mainpath+'/docs/manual.pdf')
			else:
				tkMessageBox.showwarning('No PDF-Viewer specified','You have not specified your PDF-viewer in the setup file!')
		else:
			tkMessageBox.showwarning('File Error','You have deleted the setup file!')
		
	
	def  about (self):
		tkMessageBox.showinfo('About', 'Author: Nadine Schneider\nVersion: 0.01\nDate : 12/07')	
	
	
	def callback(self): # deletes all the files produced during the session
    		if tkMessageBox.askokcancel("Quit", "Do you really wish to quit?"):

			allcreatedfiles = listdir(self.idir)
			
			for f in allcreatedfiles:
				if f != 'DejaVuSans.ttf':
					remove(self.idir+'/'+f)
			
			try:
				access(self.path+'/Memory.Log',F_OK)
				remove(self.path+'/Memory.Log')
			except OSError:
				pass
			
			self.mainwindow.destroy()

gui = Gui()

